# sierpinksy-triangle.py

from tkinter import *

def init(data):
    data.level = 1

def drawSierpinskyTriangle(canvas, x, y, size, level):
    # (x,y) is the lower-left corner of the triangle
    # size is the length of a side
    triangleHeight = size * (3**0.5)/2
    if (level == 0):
        canvas.create_polygon(x, y,
                              x + size, y,
                              x + size/2, y - triangleHeight,
                              fill = "black")
    else:
        upperX = x + size/4
        upperY = y - triangleHeight/2
        drawSierpinskyTriangle(canvas, x, y, size/2, level - 1)
        drawSierpinskyTriangle(canvas, x + size/2, y, size/2, level - 1)
        drawSierpinskyTriangle(canvas, upperX, upperY, size/2, level - 1)

def keyPressed(event, data):
    if (event.keysym in ["Up", "Right"]):
        data.level += 1
    elif ((event.keysym in ["Down", "Left"]) and (data.level > 0)):
        data.level -= 1

def redrawAll(canvas, data):
    margin = 50
    x, y = margin, data.height - margin
    size = min(data.width, data.height) - 2*margin
    drawSierpinskyTriangle(canvas, x, y, size, data.level)
    canvas.create_text(data.width/2, margin/2,
                       text = "Level %d Sierpinsky Triangle" % (data.level),
                       font = "Arial 20 bold")
    canvas.create_text(data.width/2, margin,
                       text = "Use arrows to change level",
                       font = "Arial 16")

def mousePressed(event, data): pass

def timerFired(data): pass

####################################
# use the run function as-is
####################################

def run(width=300, height=300):
    def redrawAllWrapper(canvas, data):
        canvas.delete(ALL)
        canvas.create_rectangle(0, 0, data.width, data.height,
                                fill='white', width=0)
        redrawAll(canvas, data)
        canvas.update()    

    def mousePressedWrapper(event, canvas, data):
        mousePressed(event, data)
        redrawAllWrapper(canvas, data)

    def keyPressedWrapper(event, canvas, data):
        keyPressed(event, data)
        redrawAllWrapper(canvas, data)

    def timerFiredWrapper(canvas, data):
        timerFired(data)
        redrawAllWrapper(canvas, data)
        # pause, then call timerFired again
        canvas.after(data.timerDelay, timerFiredWrapper, canvas, data)
    # Set up data and call init
    class Struct(object): pass
    data = Struct()
    data.width = width
    data.height = height
    data.timerDelay = 100 # milliseconds
    root = Tk()
    root.resizable(width=False, height=False) # prevents resizing window
    init(data)
    # create the root and the canvas
    canvas = Canvas(root, width=data.width, height=data.height)
    canvas.configure(bd=0, highlightthickness=0)
    canvas.pack()
    # set up events
    root.bind("<Button-1>", lambda event:
                            mousePressedWrapper(event, canvas, data))
    root.bind("<Key>", lambda event:
                            keyPressedWrapper(event, canvas, data))
    timerFiredWrapper(canvas, data)
    # and launch the app
    root.mainloop()  # blocks until window is closed
    print("bye!")

run(500, 500)